//
//  MyTextView.mm
//  MyTextView
//
//  Created by Jose Miguel Gomez Gonzalez
//


#include "MyTextView.h"
#include "MyTextViewBridge.h"

static MyTextViewBridge *g_MyTextView=nil;

MyTextView::MyTextView()
{
    
}

MyTextView::~MyTextView()
{
    
}

MyTextView* MyTextView::createWithRect(CCRect rect, const char* placeholder, ccColor3B color, CCNode* container)
{
    MyTextView* textView = MyTextView::create();
    textView->initWithRect(rect, placeholder, color, container);
    return textView;
}

bool MyTextView::initWithRect(CCRect rect, const char* placeholder, ccColor3B color, CCNode* container)
{
    if ( !CCLayerColor::initWithColor(ccc4(255,0,0,0), rect.size.width, rect.size.height) )
    {
        return false;
    }
    
    container->addChild(this);
    
    this->setContentSize(CCSizeMake(rect.size.width, rect.size.height));
    
    g_MyTextView = [[MyTextViewBridge alloc] init];
    
    [g_MyTextView
     initWithRect:CGRectMake(rect.origin.x, rect.origin.y, rect.size.width, rect.size.height)
     placeholder:[NSString stringWithUTF8String:placeholder]
     color:[UIColor colorWithRed:color.r/255.0f green:color.g/255.0f blue:color.b/255.0f alpha:1.0]
     container:this
     ];
    
    this->setPosition(ccp(rect.origin.x, rect.origin.y));
    prevPos = this->getParent()->convertToWorldSpace(this->getPosition());
    wasVisible = true;
    
    m_pDelegate = NULL;
    
    this->scheduleUpdate();
    
    return true;
}

void MyTextView::onEnterTransitionDidFinish()
{
    CCLayerColor::onEnterTransitionDidFinish();
}

void MyTextView::onExitTransitionDidStart()
{
    this->unscheduleUpdate();
    [g_MyTextView release];
    g_MyTextView = NULL;
    
    CCLayerColor::onExitTransitionDidStart();
}

void MyTextView::setDelegate(MyTextViewDelegate* pDelegate)
{
    m_pDelegate = pDelegate;
}

void MyTextView::setText(const char* text)
{
    [g_MyTextView setText:[NSString stringWithUTF8String:text]];
}

const char* MyTextView::getText()
{
    return [[g_MyTextView getText] UTF8String];
}


void MyTextView::textViewEditingShouldBegin(MyTextView* textView)
{
    if (m_pDelegate != NULL) m_pDelegate->textViewEditingShouldBegin(textView);
}

void MyTextView::textViewEditingDidBegin(MyTextView* textView)
{
    if (m_pDelegate != NULL) m_pDelegate->textViewEditingDidBegin(textView);
}

void MyTextView::textViewEditingShouldEnd(MyTextView* textView)
{
    if (m_pDelegate != NULL) m_pDelegate->textViewEditingShouldEnd(textView);
}

void MyTextView::textViewEditingDidEnd(MyTextView* textView)
{
    if (m_pDelegate != NULL) m_pDelegate->textViewEditingDidEnd(textView);
}

void MyTextView::textViewTextChanged(MyTextView* textView, const char* text)
{
    if (m_pDelegate != NULL) m_pDelegate->textViewTextChanged(textView, text);
}

void MyTextView::textViewKeyboardWillShow(CGSize size)
{
    if (m_pDelegate != NULL) m_pDelegate->textViewKeyboardWillShow(CCSizeMake(size.width, size.height));
}

void MyTextView::textViewKeyboardWillHide()
{
    if (m_pDelegate != NULL) m_pDelegate->textViewKeyboardWillHide();
}

void MyTextView::update(float dt)
{
    if (g_MyTextView == NULL) return;
    
    CCPoint currentPos = this->getParent()->convertToWorldSpace(this->getPosition());
    //CCLog("+++++++ %f, %f", currentPos.x, currentPos.y);
    
    if (ccpDistance(currentPos, prevPos) != 0)
    {
        [g_MyTextView updatePosition:CGPointMake(currentPos.x, currentPos.y)];
        
        prevPos = currentPos;
    }
    
    if (!this->isVisible() && wasVisible)
    {
        [g_MyTextView setVisible:false];
        wasVisible = false;
    }
    else if (this->isVisible() && !wasVisible)
    {
        [g_MyTextView setVisible:true];
        wasVisible = true;
    }
}