//
//  MyTextViewBridge.cpp
//  MyTextView
//
//  Created by Jose Miguel Gomez Gonzalez
//

#import "MyTextViewBridge.h"
#import "EAGLView.h"
#include "GlobalDefines.h"

@implementation MyTextViewBridge

- (id)init
{
    self = [super init];
    
    if (self) {
        // init code here.
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIKeyboardWillShowNotification
                                                  object:nil];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIKeyboardWillHideNotification
                                                  object:nil];
    
    [mPlaceholder release];
    [mtextView removeFromSuperview];
    [mtextView release];
    [super dealloc];
    
}

-(void) initWithRect:(CGRect)rect placeholder:(NSString *)placeholder color:(UIColor*)color container:(MyTextView *)container
{
    size = CCDirector::sharedDirector()->getWinSize();
    factor = size.width == 640 ? 2.0f : 1.0f;
    
    mtextView = [[UITextView alloc] initWithFrame:CGRectMake(rect.origin.x/factor, (size.height - rect.origin.y - rect.size.height)/factor, rect.size.width/factor, rect.size.height/factor)];
    
    originalPos = CGPointMake(rect.origin.x/factor, rect.origin.y/factor);
    
    mPlaceholder = placeholder;
    [mPlaceholder retain];
    
    mMyTextView = container;
    
    //mtextView.backgroundColor = [UIColor redColor];
    mtextView.backgroundColor = [UIColor clearColor];
    
    //[mtextView setFont: [ UIFont systemFontOfSize:FONT_SIZE_INPUT_SMALL/factor]];
    [mtextView setFont: [UIFont fontWithName:@FONT_NAME_REGULAR size:FONT_SIZE_INPUT_SMALL/factor]];
    
    [mtextView setText:mPlaceholder];
    [mtextView setTextColor:color];
    
    mtextView.delegate = self;
    
    [mtextView setReturnKeyType:UIReturnKeyDone];
    //[mtextView becomeFirstResponder]; // Opens keyboard
    
    [[EAGLView sharedEGLView] addSubview: mtextView];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(textViewKeyboardWillShow:)
                                                 name:UIKeyboardWillShowNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(textViewKeyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
}

- (void)updatePosition:(CGPoint)pos
{
    mtextView.frame = CGRectMake(pos.x/factor, (size.height - pos.y - mtextView.frame.size.height*factor)/factor, mtextView.frame.size.width, mtextView.frame.size.height);
}

- (BOOL)textViewShouldBeginEditing:(UITextView *)textView
{
    if (originalPos.x != mtextView.frame.origin.x) return false; // Don't open keyboard if not on screen (we assume it hides horizontally)
    
    if ([[textView text] isEqualToString:mPlaceholder]) [textView setText:@""];
    
    mMyTextView->textViewEditingShouldBegin(mMyTextView);
    
    return YES;
}

- (void)textViewDidBeginEditing:(UITextView *)textView
{
    mMyTextView->textViewEditingDidBegin(mMyTextView);
}

- (BOOL)textViewShouldEndEditing:(UITextView *)textView
{
    mMyTextView->textViewEditingShouldEnd(mMyTextView);
    
    return YES;
}

- (void)textViewDidEndEditing:(UITextView *)textView
{
    if ([[textView text] isEqualToString:@""]) [textView setText:mPlaceholder];
    
    mMyTextView->textViewEditingDidEnd(mMyTextView);
    
    [textView resignFirstResponder];
}

- (void)textViewKeyboardWillShow:(NSNotification *)notification
{
    NSDictionary* userInfo = [notification userInfo];
    
    // get the size of the keyboard
    CGSize keyboardSize = [[userInfo objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;

    mMyTextView->textViewKeyboardWillShow(CGSizeMake(keyboardSize.width*factor, keyboardSize.height*factor));
}

- (void)textViewKeyboardWillHide:(NSNotification *)notification
{
    mMyTextView->textViewKeyboardWillHide();
}

- (void)textViewDidChangeSelection:(UITextView *)textView
{
}

-(BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text
{
    if ([text isEqualToString:@"\n"]) {
        // Be sure to test for equality using the "isEqualToString" message
        [textView resignFirstResponder];
        
        // Return FALSE so that the final '\n' character doesn't get added
        return FALSE;
    }
    
    mMyTextView->textViewTextChanged(mMyTextView, [text UTF8String]);
    
    // For any other character return TRUE so that the text gets added to the view
    return TRUE;
}

- (void)setVisible:(BOOL)visible
{
    [mtextView setHidden:!visible];
}

- (void)setText:(NSString*)text
{
    [mtextView setText:text];
}

- (NSString*)getText
{
    return [mtextView text];
}


@end